@@ -78,7 +78,7 @@ static struct db *newsrc_db = NULL;
 static int newsrc_dbopen = 0;
 
 /* must be called after cyrus_init */
-int newsrc_init(const char *fname, int myflags __attribute__((unused)))
+int newsrc_init(char *fname, int myflags __attribute__((unused)))
 {
     char buf[1024];
     int r = 0;
@@ -89,13 +89,12 @@ int newsrc_init(const char *fname, int myflags __attribute__((unused)))
     else {
 	char *tofree = NULL;
 
-	if (!fname)
-	    fname = config_getstring(IMAPOPT_NEWSRC_DB_PATH);
-
 	/* create db file name */
 	if (!fname) {
-	    tofree = strconcat(config_dir, FNAME_NEWSRCDB, (char *)NULL);
-	    fname = tofree;
+	    fname = xmalloc(strlen(config_dir)+sizeof(FNAME_NEWSRCDB));
+	    tofree = fname;
+	    strcpy(fname, config_dir);
+	    strcat(fname, FNAME_NEWSRCDB);
 	}
 
 	r = (DB->open)(fname, CYRUSDB_CREATE, &newsrc_db);
@@ -105,7 +104,7 @@ int newsrc_init(const char *fname, int myflags __attribute__((unused)))
 	else
 	    newsrc_dbopen = 1;
 
-	free(tofree);
+	if (tofree) free(tofree);
     }
 
     return r;
@@ -353,7 +352,7 @@ int main(int argc, char *argv[])
 	else if (!strncmp("381", buf, 3)) {
 	    /* password required */
 	    if (!password)
-		password = cyrus_getpass("Please enter the password: ");
+		password = getpass("Please enter the password: ");
 
 	    if (!password) {
 		fprintf(stderr, "failed to get password\n");
