@@ -419,11 +419,14 @@ int handle_request(const char *who, const char *name,
 	lastread = mailbox->i.recenttime;
 	recentuid = mailbox->i.recentuid;
     } else {
-	struct seen *seendb = NULL;
+	struct seen *seendb;
 	struct seendata sd;
 	r = seen_open(who, 0, &seendb);
-	if (!r) r = seen_read(seendb, mailbox->uniqueid, &sd);
-	seen_close(&seendb);
+
+	if (!r) {
+	    r = seen_read(seendb, mailbox->uniqueid, &sd);
+	    seen_close(seendb);
+	}
 
 	if (r) {
 	    /* Fake Data -- couldn't open seen database */
@@ -451,7 +454,7 @@ int handle_request(const char *who, const char *name,
     }
 
     mailbox_close(&mailbox);
-
+    
     send_reply(sfrom, sfromsiz, REQ_OK, who, name, numrecent,
 	       lastread, lastarrived);
     
@@ -471,9 +474,9 @@ send_reply(struct sockaddr_storage sfrom, socklen_t sfromsiz,
             sendto(soc,"PERMDENY",9,0,(struct sockaddr *) &sfrom, sfromsiz);       
             break;
         case REQ_OK:
-            siz = snprintf(buf, sizeof(buf), "%s|%s|%d|%u|%u",
-			   user, mbox, numrecent,
-			   (uint32_t)lastread, (uint32_t)lastarrived);
+            siz = snprintf(buf, sizeof(buf), "%s|%s|%d|%d|%d",
+			   user,mbox,numrecent,(int) lastread,
+			   (int)lastarrived);
             sendto(soc,buf,siz,0,(struct sockaddr *) &sfrom, sfromsiz);       
             break;
         case REQ_UNK:
