@@ -1,4 +1,4 @@
-/* mboxlist.c -- Mailbox list manipulation routines
+/* uboxlist.c -- Mailbox list manipulation routines
  *
  * Copyright (c) 1994-2008 Carnegie Mellon University.  All rights reserved.
  *
@@ -64,7 +64,6 @@
 #include "acl.h"
 #include "annotate.h"
 #include "auth.h"
-#include "bsearch.h"
 #include "glob.h"
 #include "assert.h"
 #include "global.h"
@@ -176,12 +175,9 @@ static int mboxlist_mylookup(const char *name, struct mboxlist_entry *entry,
     /* copy out interesting parts */
     mbtype = strtol(data, &p, 10);
 
-    if (!p)
-	return IMAP_IOERROR;
-
-    if (*p == ' ' && (p - data) < datalen) p++;
+    if (*p == ' ') p++;
     q = partition;
-    while (*p != ' ' && (p - data) < datalen) { /* copy out partition name */
+    while (*p != ' ') { /* copy out partition name */
 	*q++ = *p++;
     }
     *q = '\0';
@@ -559,8 +555,7 @@ int mboxlist_createmailbox_full(const char *name, int mbtype,
 				struct auth_state *auth_state,
 				int options, unsigned uidvalidity,
 				const char *copyacl, const char *uniqueid,
-				int localonly, int forceuser, int dbonly,
-				struct mailbox **mboxptr)
+				int localonly, int forceuser, int dbonly)
 {
     int r;
     char *newpartition = NULL;
@@ -582,16 +577,11 @@ int mboxlist_createmailbox_full(const char *name, int mbtype,
 	/* 2. verify ACL's to best of ability (CRASH: abort) */
 	r = mboxlist_mycreatemailboxcheck(name, mbtype, partition, isadmin, 
 					  userid, auth_state, 
-					  &acl, &newpartition, 0, localonly,
+					  &acl, &newpartition, 1, localonly,
 					  forceuser, NULL);
 	if (r) goto done;
     }
 
-    if (!newpartition) {
-	r = IMAP_IOERROR;
-	goto done;
-    }
-
     if (!dbonly && !isremote) {
 	/* Filesystem Operations */
 	r = mailbox_create(name, newpartition, acl, uniqueid,
@@ -629,7 +619,6 @@ int mboxlist_createmailbox_full(const char *name, int mbtype,
 done:
     if (newmailbox) {
 	if (r) mailbox_delete(&newmailbox);
-	else if (mboxptr) *mboxptr = newmailbox;
 	else mailbox_close(&newmailbox);
     }
 
@@ -651,20 +640,19 @@ int mboxlist_createmailbox(const char *name, int mbtype,
     return mboxlist_createmailbox_full(name, mbtype, partition,
 				       isadmin, userid, auth_state,
 				       options, uidvalidity, NULL, NULL,
-				       localonly, forceuser, dbonly, NULL);
+				       localonly, forceuser, dbonly);
 }
 
 int mboxlist_createsync(const char *name, int mbtype,
 			const char *partition,
 			const char *userid, struct auth_state *auth_state,
 			int options, unsigned uidvalidity,
-			const char *acl, const char *uniqueid,
-			struct mailbox **mboxptr)
+			const char *acl, const char *uniqueid)
 {
     return mboxlist_createmailbox_full(name, mbtype, partition,
 				       1, userid, auth_state,
 				       options, uidvalidity, acl, uniqueid,
-				       0, 1, 0, mboxptr);
+				       0, 1, 0);
 }
 
 /* insert an entry for the proxy */
@@ -850,12 +838,7 @@ mboxlist_delayed_deletemailbox(const char *name, int isadmin,
     r = mboxlist_renamemailbox((char *)name, newname, mbentry.partition,
                                1 /* isadmin */, userid,
                                auth_state, force, 1);
-    if (r) goto out;
 
-    /* Rename mailbox annotations */
-    r = annotatemore_rename(name, newname, NULL, NULL);
-
-out:
     return r;
 }
 
@@ -997,10 +980,11 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
     int partitionmove = 0;
     struct mailbox *oldmailbox = NULL;
     struct mailbox *newmailbox = NULL;
-    struct txn *tid = NULL;
+    struct txn *tid;
     const char *root = NULL;
     char *newpartition = NULL;
     char *mboxent = NULL;
+    char *p;
     mupdate_handle *mupdate_h = NULL;
 
     /* 1. open mailbox */
@@ -1009,37 +993,18 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
 
     myrights = cyrus_acl_myrights(auth_state, oldmailbox->acl);
 
-    /* check the ACLs up-front */
-    if (!isadmin) {
-	if (!(myrights & ACL_DELETEMBOX)) {
-	    r = (myrights & ACL_LOOKUP) ?
-		IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
-	    goto done;
-	}
-    }
-
-    /* 2. verify valid move */
+    /* 2. verify acls */
     /* XXX - handle remote mailbox */
-
-    /* special case: same mailbox, must be a partition move */
     if (!strcmp(oldname, newname)) {
 	char *oldpath = mailbox_datapath(oldmailbox);
 
-	/* Only admin can move mailboxes between partitions */
+	/* Attempt to move mailbox across partition */
 	if (!isadmin) {
 	    r = IMAP_PERMISSION_DENIED;
 	    goto done;
 	}
 
-	/* No partition, we're definitely not moving anywhere */
-	if (!partition) {
-	    r = IMAP_MAILBOX_EXISTS;
-	    goto done;
-	}
-
-	/* let mupdate code below know it was a partition move */
 	partitionmove = 1;
-
 	/* this is OK because it uses a different static buffer */
 	root = config_partitiondir(partition);
 	if (!root) {
@@ -1052,84 +1017,87 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
 	    r = IMAP_MAILBOX_EXISTS;
 	    goto done;
 	}
-
-	/* NOTE: this is a rename to the same mailbox name on a
-	 * different partition.  This is a pretty filthy hack,
-	 * which should be handled by having four totally different
-	 * codepaths: INBOX -> INBOX.foo, user rename, regular rename
-	 * and of course this one, partition move */
-	newpartition = xstrdup(partition);
-	r = mailbox_copy_files(oldmailbox, newpartition, newname);
-	if (r) goto done;
-	mboxent = mboxlist_makeentry(oldmailbox->mbtype, newpartition,
-				     oldmailbox->acl);
-	r = DB->store(mbdb, newname, strlen(newname), 
-		      mboxent, strlen(mboxent), &tid);
-	if (r) goto done;
-
-	/* skip ahead to the commit */
-	goto dbdone;
-    }
-
-    /* RENAME of some user's INBOX */
-    if (mboxname_isusermailbox(oldname, 1)) {
-	if (mboxname_isdeletedmailbox(newname)) {
-	    /* delete user is OK */
-	}
-	else if (mboxname_isusermailbox(newname, 1)) {
-	    /* user rename is depends on config */
-	    if (!config_getswitch(IMAPOPT_ALLOWUSERMOVES)) {
-		r = IMAP_MAILBOX_NOTSUPPORTED;
+    } else if ((p = mboxname_isusermailbox(oldname, 1))) {
+	if (!strncmp(p, userid, config_virtdomains ? strcspn(userid, "@") :
+		     strlen(userid))) {
+	    /* Special case of renaming inbox */
+	    if (!(myrights & ACL_DELETEMBOX)) {
+		r = IMAP_PERMISSION_DENIED;
 		goto done;
 	    }
-	}
-	else if (mboxname_userownsmailbox(userid, oldname) &&
-		 mboxname_userownsmailbox(userid, newname)) {
-	    /* Special case of renaming inbox */
 	    isusermbox = 1;
-	}
-	else {
-	    /* Everything else is bogus */
+	} else if ((config_getswitch(IMAPOPT_ALLOWUSERMOVES) &&
+		    mboxname_isusermailbox(newname, 1)) ||
+		   mboxname_isdeletedmailbox(newname)) {
+	    /* Special case of renaming a user */
+	    if (!(myrights & ACL_DELETEMBOX) && !isadmin) {
+		r = (myrights & ACL_LOOKUP) ?
+		    IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
+		goto done;
+	    }
+	} else {
+	    /* Even admins can only rename an INBOX to another INBOX */
 	    r = IMAP_MAILBOX_NOTSUPPORTED;
 	    goto done;
 	}
+    } else {
+	if (!(myrights & ACL_DELETEMBOX) && !isadmin) {
+	    r = (myrights & ACL_LOOKUP) ?
+		IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
+	    goto done;
+	}
     }
 
-    r = mboxlist_mycreatemailboxcheck(newname, 0, partition, isadmin, 
-				      userid, auth_state, NULL, 
-				      &newpartition, 1, 0, forceuser, NULL);
-    if (r) goto done;
-
-    if (!newpartition) newpartition = xstrdup(config_defpartition);
+    /* Check ability to create new mailbox */
+    if (!partitionmove) {
+	if (mboxname_isusermailbox(newname, 1)) {
+	    if ((config_getswitch(IMAPOPT_ALLOWUSERMOVES) &&
+		 mboxname_isusermailbox(oldname, 1)) ||
+		mboxname_isdeletedmailbox(oldname)) {
+		if (!isadmin) {
+		    /* Only admins can rename users (INBOX to INBOX) */
+		    r = IMAP_MAILBOX_NOTSUPPORTED;
+		    goto done;
+		}
+	    } else {
+		/* Even admins can't rename to user's inboxes */
+		r = IMAP_MAILBOX_NOTSUPPORTED;
+		goto done;
+	    }
+	}
+	r = mboxlist_mycreatemailboxcheck(newname, 0, partition, isadmin, 
+					  userid, auth_state, NULL, 
+					  &newpartition, 1, 0, forceuser, NULL);
+	if (r) goto done;
+    }
+    else if (partition) {
+	newpartition = xstrdup(partition);
+    }
 
     /* Rename the actual mailbox */
-    syslog(LOG_INFO, "Rename: %s -> %s", oldname, newname);
-
     r = mailbox_rename_copy(oldmailbox, newname, newpartition,
 			    isusermbox ? userid : NULL, ignorequota,
 			    &newmailbox);
     if (r) goto done;
 
-    /* create new entry */
+    /* 7a. create new entry */
     mboxent = mboxlist_makeentry(newmailbox->mbtype, newmailbox->part,
 				 newmailbox->acl);
     do {
-	r = 0;
+	tid = NULL;
+	/* 7b. put it into the db */
+	r = DB->store(mbdb, newname, strlen(newname), 
+		      mboxent, strlen(mboxent), &tid);
 
-	/* delete the old entry */
-	if (!isusermbox)
+	if (!r && !isusermbox) {
+	    /* 4. Delete entry from berkeley db */
 	    r = DB->delete(mbdb, oldname, strlen(oldname), &tid, 0);
-
-	/* create a new entry */
-	if (!r)
-	    r = DB->store(mbdb, newname, strlen(newname), 
-			  mboxent, strlen(mboxent), &tid);
+	}
 
 	switch (r) {
 	case 0: /* success */
 	    break;
 	case CYRUSDB_AGAIN:
-	    tid = NULL;
 	    break;
 	default:
 	    syslog(LOG_ERR, "DBERROR: error renaming %s %s: %s",
@@ -1140,8 +1108,6 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
 	}
     } while (r == CYRUSDB_AGAIN);
 
- dbdone:
-
     /* 3. Commit transaction */
     r = DB->commit(mbdb, tid);
     tid = NULL;
@@ -1155,7 +1121,6 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
     if (config_mupdate_server) {
 	/* commit the mailbox in MUPDATE */
 	char buf[MAX_PARTITION_LEN + HOSTNAME_SIZE + 2];
-	const char *acl = newmailbox ? newmailbox->acl : oldmailbox->acl;
 
 	snprintf(buf, sizeof(buf), "%s!%s",
 		 config_servername, newpartition);
@@ -1166,7 +1131,7 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
 		r = mupdate_delete(mupdate_h, oldname);
 	    if (!r) r = mupdate_reserve(mupdate_h, newname, buf);
 	}
-	if (!r) r = mupdate_activate(mupdate_h, newname, buf, acl);
+	if (!r) r = mupdate_activate(mupdate_h, newname, buf, newmailbox->acl);
 	if (r) {
 	    syslog(LOG_ERR,
 		   "MUPDATE: can't commit mailbox entry for '%s'",
@@ -1176,36 +1141,13 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
     }
 
  done: /* Commit or cleanup */
-    if (!r && newmailbox)
-	r = mailbox_commit(newmailbox);
-
     if (r) {
 	/* XXX - rollback DB changes if it was an mupdate failure */
 	if (newmailbox) mailbox_delete(&newmailbox);
-	if (partitionmove && newpartition)
-	    mailbox_delete_cleanup(newpartition, newname);
 	mailbox_close(&oldmailbox);
     } else {
-	if (newmailbox) {
-	    mailbox_rename_cleanup(&oldmailbox, isusermbox);
-	    mailbox_close(&newmailbox);
-	}
-	else if (partitionmove) {
-	    char *oldpartition = xstrdup(oldmailbox->part);
-	    if (config_auditlog)
-		syslog(LOG_NOTICE, "auditlog: partitionmove sessionid=<%s> "
-		       "mailbox=<%s> uniqueid=<%s> oldpart=<%s> newpart=<%s>",
-		       session_id(),
-		       oldmailbox->name, oldmailbox->uniqueid,
-		       oldpartition, partition);
-	    mailbox_close(&oldmailbox);
-	    mailbox_delete_cleanup(oldpartition, oldname);
-	    free(oldpartition);
-
-	}
-	else
-	    abort(); /* impossible, in theory */
-
+	mailbox_close(&newmailbox);
+	mailbox_rename_cleanup(&oldmailbox, isusermbox);
 	/* log the rename */
 	sync_log_mailbox_double(oldname, newname);
     }
@@ -1396,7 +1338,7 @@ int mboxlist_setacl(const char *name, const char *identifier,
     /* 4. Change backup copy (cyrus.header) */
     /* we already have it locked from above */
     if (!r && !(mbentry.mbtype & MBTYPE_REMOTE)) {
-	mailbox_set_acl(mailbox, newacl, 1);
+	mailbox_set_acl(mailbox, newacl);
 	/* want to commit immediately to ensure ordering */
 	r = mailbox_commit(mailbox);
     }
@@ -1561,8 +1503,6 @@ struct find_rock {
     int checkshared;
     int isadmin;
     struct auth_state *auth_state;
-    char *prev;
-    int prevlen;
     int (*proc)(char *, int, int, void *rock);
     void *procrock;
 };
@@ -1632,7 +1572,8 @@ static int find_p(void *rockp,
 
 	memcpy(namebuf, key, keylen);
 	namebuf[keylen] = '\0';
-	if (mboxname_isdeletedmailbox(namebuf))
+	if (mboxlist_delayed_delete_isenabled() && 
+	    mboxname_isdeletedmailbox(namebuf))
 	    return 0;
     }
 
@@ -1677,25 +1618,7 @@ static int find_p(void *rockp,
     return 1;
 }
 
-static int check_name(struct find_rock *rock,
-		      const char *base, int len)
-{
-    if (rock->prev) {
-	/* check if name is not a substring of the last sent.  Note,
-	 * you actually have to check 1 character PAST the length,
-	 * so you don't forget to tell about subfolders. */
-	if (len <= rock->prevlen && !memcmp(base, rock->prev, len+1))
-	    return 0;
-	free(rock->prev);
-    }
-
-    rock->prev = xstrndup(base, len);
-    rock->prevlen = len;
-
-    return 1;
-}
-
-static int find_cb(void *rockp,
+static int find_cb(void *rockp, 
 		   const char *key, int keylen,
 		   const char *data __attribute__((unused)),
 		   int datalen __attribute__((unused)))
@@ -1710,14 +1633,14 @@ static int find_cb(void *rockp,
     minmatch = 0;
     while (minmatch >= 0) {
 	long matchlen;
-
+	
 	if(keylen >= (int) sizeof(namebuf)) {
 	    syslog(LOG_ERR, "oversize keylen in mboxlist.c:find_cb()");
 	    return 0;
 	}
 	memcpy(namebuf, key, keylen);
 	namebuf[keylen] = '\0';
-
+	
 	if (rock->find_namespace != NAMESPACE_INBOX &&
 	    rock->usermboxname &&
 	    !strncmp(namebuf, rock->usermboxname, rock->usermboxnamelen)
@@ -1727,7 +1650,7 @@ static int find_cb(void *rockp,
 	    return 0;
 	}
 
-	/* make sure it's in the mailboxes db */
+      	/* make sure it's in the mailboxes db */
 	if (rock->checkmboxlist) {
 	    r = mboxlist_lookup(namebuf, NULL, NULL);
 	} else {
@@ -1741,7 +1664,7 @@ static int find_cb(void *rockp,
 	    namebuf[rock->inboxoffset+3] = rock->inboxcase[3];
 	    namebuf[rock->inboxoffset+4] = rock->inboxcase[4];
 	}
-
+	
 	matchlen = glob_test(g, namebuf+rock->inboxoffset,
 			     keylen-rock->inboxoffset, &minmatch);
 
@@ -1768,13 +1691,11 @@ static int find_cb(void *rockp,
 	    }
 
 	    rock->checkshared = 0;
-
-	    if (check_name(rock, namebuf+rock->inboxoffset, matchlen))
-		r = (*rock->proc)(namebuf+rock->inboxoffset, matchlen,
-				  1, rock->procrock);
+	    r = (*rock->proc)(namebuf+rock->inboxoffset, matchlen, 
+			      1, rock->procrock);
 
 	    break;
-
+	    
 	case IMAP_MAILBOX_NONEXISTENT:
 	    /* didn't find the entry */
 	    r = 0;
@@ -1790,16 +1711,6 @@ static int find_cb(void *rockp,
     return r;
 }
 
-int mboxlist_allmbox(const char *prefix, foreach_cb *proc, void *rock)
-{
-    int r;
-    char *search = prefix ? (char *)prefix : "";
-
-    r = DB->foreach(mbdb, search, strlen(search), NULL, proc, rock, 0);
-
-    return r;
-}
-
 /*
  * Find all mailboxes that match 'pattern'.
  * 'isadmin' is nonzero if user is a mailbox admin.  'userid'
@@ -1810,7 +1721,7 @@ int mboxlist_allmbox(const char *prefix, foreach_cb *proc, void *rock)
  * case it wants some persistant storage or extra data.
  */
 /* Find all mailboxes that match 'pattern'. */
-int mboxlist_findall(struct namespace *namespace,
+int mboxlist_findall(struct namespace *namespace __attribute__((unused)),
 		     const char *pattern, int isadmin, const char *userid, 
 		     struct auth_state *auth_state, int (*proc)(), void *rock)
 {
@@ -1826,8 +1737,6 @@ int mboxlist_findall(struct namespace *namespace,
     char domainpat[MAX_MAILBOX_BUFFER] = ""; /* do intra-domain fetches only */
     char *pat = NULL;
 
-    if (!namespace) namespace = mboxname_get_adminnamespace();
-
     if (config_virtdomains) {
 	char *domain;
 
@@ -1877,8 +1786,6 @@ int mboxlist_findall(struct namespace *namespace,
     cbrock.checkshared = 0;
     cbrock.proc = proc;
     cbrock.procrock = rock;
-    cbrock.prev = NULL;
-    cbrock.prevlen = 0;
 
     /* Build usermboxname */
     if (userid && (!(p = strchr(userid, '.')) || ((p - userid) > userlen)) &&
@@ -1960,13 +1867,9 @@ int mboxlist_findall(struct namespace *namespace,
 			usermboxname, usermboxnamelen,
 			&find_p, &find_cb, &cbrock,
 			NULL);
-
-	free(cbrock.prev);
-	cbrock.prev = NULL;
-	cbrock.prevlen = 0;
     }
 
-    if (!r && (isadmin || namespace->accessible[NAMESPACE_USER])) {
+    if(!r) {
 	cbrock.find_namespace = NAMESPACE_USER;
 	/* switch to pattern with domain prepended */
 	glob_free(&cbrock.g);
@@ -1984,11 +1887,8 @@ int mboxlist_findall(struct namespace *namespace,
 			domainpat, domainlen + prefixlen,
 			&find_p, &find_cb, &cbrock,
 			NULL);
-
-	free(cbrock.prev);
-	cbrock.prev = NULL;
-	cbrock.prevlen = 0;
     }
+    
 
   done:
     glob_free(&cbrock.g);
@@ -2014,8 +1914,6 @@ int mboxlist_findall_alt(struct namespace *namespace,
     char domainpat[MAX_MAILBOX_BUFFER]; /* do intra-domain fetches only */
     char *pat = NULL;
 
-    if (!namespace) namespace = mboxname_get_adminnamespace();
-
     if (config_virtdomains && userid && (p = strchr(userid, '@'))) {
 	userlen = p - userid;
 	domainlen = strlen(p); /* includes separator */
@@ -2034,8 +1932,6 @@ int mboxlist_findall_alt(struct namespace *namespace,
     cbrock.checkshared = 0;
     cbrock.proc = proc;
     cbrock.procrock = rock;
-    cbrock.prev = NULL;
-    cbrock.prevlen = 0;
 
     /* Build usermboxname */
     if (userid && (!(p = strchr(userid, '.')) || ((p - userid) > userlen)) &&
@@ -2091,10 +1987,11 @@ int mboxlist_findall_alt(struct namespace *namespace,
      * Append pattern to "INBOX.", search for those mailboxes next
      */
     if (userid) {
-	strlcpy(patbuf, usermboxname, sizeof(patbuf));
+	strlcpy(patbuf, "INBOX.", sizeof(patbuf));
 	strlcat(patbuf, pattern, sizeof(patbuf));
-	cbrock.g = glob_init(patbuf, GLOB_HIERARCHY);
-	cbrock.inboxoffset = 0;
+	cbrock.g = glob_init(patbuf, GLOB_HIERARCHY|GLOB_INBOXCASE);
+	cbrock.inboxcase = glob_inboxcase(cbrock.g);
+	cbrock.inboxoffset = domainlen+userlen;
 	cbrock.find_namespace = NAMESPACE_INBOX;
 
 	/* iterate through prefixes matching usermboxname */
@@ -2103,9 +2000,6 @@ int mboxlist_findall_alt(struct namespace *namespace,
 		    &find_p, &find_cb, &cbrock,
 		    NULL);
 
-	free(cbrock.prev);
-	cbrock.prev = NULL;
-	cbrock.prevlen = 0;
 	glob_free(&cbrock.g);
     }
 
@@ -2120,38 +2014,33 @@ int mboxlist_findall_alt(struct namespace *namespace,
      *
      * If "Other Users*" can match pattern, search for those mailboxes next
      */
-    if (isadmin || namespace->accessible[NAMESPACE_USER]) {
-        len = strlen(namespace->prefix[NAMESPACE_USER]);
-        if(len>0) len--;
+    len = strlen(namespace->prefix[NAMESPACE_USER]);
+    if(len>0) len--;
 
-	if (!strncmp(namespace->prefix[NAMESPACE_USER], pattern,
+    if (!strncmp(namespace->prefix[NAMESPACE_USER], pattern,
 		 prefixlen < len ? prefixlen : len)) {
 
-	    if (prefixlen < len) {
-	        strlcpy(domainpat+domainlen, pattern+prefixlen,
+	if (prefixlen < len) {
+	    strlcpy(domainpat+domainlen, pattern+prefixlen,
 		    sizeof(domainpat)-domainlen);
-	        cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
-	    }
-	    else {
-	        strlcpy(domainpat+domainlen, "user", sizeof(domainpat)-domainlen);
-	        strlcat(domainpat, pattern+len, sizeof(domainpat));
-	        cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
-	    }
-	    cbrock.find_namespace = NAMESPACE_USER;
-	    cbrock.inboxoffset = 0;
-
-	    /* iterate through prefixes matching usermboxname */
+	    cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
+	}
+	else {
 	    strlcpy(domainpat+domainlen, "user", sizeof(domainpat)-domainlen);
-	    DB->foreach(mbdb,
-			domainpat, strlen(domainpat),
-			&find_p, &find_cb, &cbrock,
-			NULL);
+	    strlcat(domainpat, pattern+len, sizeof(domainpat));
+	    cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
+	}
+	cbrock.find_namespace = NAMESPACE_USER;
+	cbrock.inboxoffset = 0;
+	
+	/* iterate through prefixes matching usermboxname */
+	strlcpy(domainpat+domainlen, "user", sizeof(domainpat)-domainlen);
+	DB->foreach(mbdb,
+		    domainpat, strlen(domainpat),
+		    &find_p, &find_cb, &cbrock,
+		    NULL);
 
-	    glob_free(&cbrock.g);
-	    free(cbrock.prev);
-	    cbrock.prev = NULL;
-	    cbrock.prevlen = 0;
-        }
+	glob_free(&cbrock.g);
     }
 
     /*
@@ -2160,59 +2049,55 @@ int mboxlist_findall_alt(struct namespace *namespace,
      * search for all remaining mailboxes.
      * just bother looking at the ones that have the same pattern prefix.
      */
-    if (isadmin || namespace->accessible[NAMESPACE_SHARED]) {
-	len = strlen(namespace->prefix[NAMESPACE_SHARED]);
-	if(len>0) len--;
-	if (!strncmp(namespace->prefix[NAMESPACE_SHARED], pattern,
-		prefixlen < len ? prefixlen : len)) {
+    len = strlen(namespace->prefix[NAMESPACE_SHARED]);
+    if(len>0) len--;
+    if (!strncmp(namespace->prefix[NAMESPACE_SHARED], pattern,
+		 prefixlen < len ? prefixlen : len)) {
 
-	    cbrock.find_namespace = NAMESPACE_SHARED;
-	    cbrock.inboxoffset = 0;
+	cbrock.find_namespace = NAMESPACE_SHARED;
+	cbrock.inboxoffset = 0;
 
-	    if (prefixlen <= len) {
-		/* Skip pattern which matches shared namespace prefix */
-		for (p = pat+prefixlen; *p; p++) {
-		    if (*p == '%') continue;
-		    else if (*p == '.') p++;
-		    break;
-		}
+	if (prefixlen <= len) {
+	    /* Skip pattern which matches shared namespace prefix */
+	    for (p = pat+prefixlen; *p; p++) {
+		if (*p == '%') continue;
+		else if (*p == '.') p++;
+		break;
+	    }
 
-		if (*pattern && !strchr(pattern, '.') &&
-		    pattern[strlen(pattern)-1] == '%')
+	    if (*pattern && !strchr(pattern, '.') &&
+		pattern[strlen(pattern)-1] == '%') {
 		/* special case:  LIST "" *% -- output prefix */
-		    cbrock.checkshared = 1;
+  		cbrock.checkshared = 1;
+  	    }
 
-		if ((cbrock.checkshared || prefixlen == len) && !*p) {
-		    /* special case:  LIST "" % -- output prefix
-		       (if we have a shared mbox) and quit */
-		    strlcpy(domainpat+domainlen, "*", sizeof(domainpat)-domainlen);
-		    cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
-		    cbrock.checkshared = 2;
-		}
-		else {
-		    strlcpy(domainpat+domainlen, p, sizeof(domainpat)-domainlen);
-		    cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
-		}
-
-		domainpat[domainlen] = '\0';
-		DB->foreach(mbdb,
-			    domainpat, domainlen,
-			    &find_p, &find_cb, &cbrock,
-			    NULL);
-	    }
-	    else if (pattern[len] == '.') {
-		strlcpy(domainpat+domainlen, pattern+len+1,
-			sizeof(domainpat)-domainlen);
+	    if ((cbrock.checkshared || prefixlen == len) && !*p) {
+		/* special case:  LIST "" % -- output prefix
+ 		   (if we have a shared mbox) and quit */
+		strlcpy(domainpat+domainlen, "*", sizeof(domainpat)-domainlen);
 		cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
-
-		DB->foreach(mbdb,
-			    domainpat, domainlen+prefixlen-(len+1),
-			    &find_p, &find_cb, &cbrock,
-			NULL);
+		cbrock.checkshared = 2;
 	    }
-	    free(cbrock.prev);
-	    cbrock.prev = NULL;
-	    cbrock.prevlen = 0;
+	    else {
+		strlcpy(domainpat+domainlen, p, sizeof(domainpat)-domainlen);
+		cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
+	    }
+		
+	    domainpat[domainlen] = '\0';
+	    DB->foreach(mbdb,
+			domainpat, domainlen,
+			&find_p, &find_cb, &cbrock,
+			NULL);
+	}
+	else if (pattern[len] == '.') {
+	    strlcpy(domainpat+domainlen, pattern+len+1,
+		    sizeof(domainpat)-domainlen);
+	    cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
+
+	    DB->foreach(mbdb,
+			domainpat, domainlen+prefixlen-(len+1),
+			&find_p, &find_cb, &cbrock,
+			NULL);
 	}
     }
 
@@ -2258,13 +2143,15 @@ int mboxlist_setquota(const char *root, int newquota, int force)
 	    q.limit = newquota;
 	    r = quota_write(&q, &tid);
 	}
-	if (!r)
-	    quota_commit(&tid);
+	if (!r) quota_commit(&tid);
+	else quota_abort(&tid);
 	goto done;
     }
 
-    if (r != IMAP_QUOTAROOT_NONEXISTENT)
+    if (r != IMAP_QUOTAROOT_NONEXISTENT) {
+	if (tid) quota_abort(&tid);
 	goto done;
+    }
 
     /*
      * Have to create a new quota root
@@ -2290,36 +2177,38 @@ int mboxlist_setquota(const char *root, int newquota, int force)
 
 	    /* are we going to force the create anyway? */
 	    if (!force) goto done;
-	    have_mailbox = 0;
-	}
-	else {
-	    if (mbentry.mbtype & (MBTYPE_REMOTE | MBTYPE_MOVING)) {
-		/* Can't set quota on a remote mailbox */
-		r = IMAP_MAILBOX_NOTSUPPORTED;
-		goto done;
+	    else {
+		have_mailbox = 0;
+		mbentry.mbtype = 0;
 	    }
 	}
+
+	if (mbentry.mbtype & (MBTYPE_REMOTE | MBTYPE_MOVING)) {
+	    /* Can't set quota on a remote mailbox */
+	    r = IMAP_MAILBOX_NOTSUPPORTED;
+	    goto done;
+	}
     }
 
     /* initialise the quota */
     q.used = 0;
     q.limit = newquota;
     r = quota_write(&q, &tid);
-    if (r) goto done;
-
-    quota_commit(&tid);
+    if (!r) quota_commit(&tid);
+    else quota_abort(&tid);
 
     /* recurse through mailboxes, setting the quota and finding
      * out the usage */
-    /* top level mailbox */
-    if (have_mailbox)
-	mboxlist_changequota(root, 0, 0, (void *)root);
+    if (!r) {
+	/* top level mailbox */
+	if (have_mailbox)
+	    mboxlist_changequota(root, 0, 0, (void *)root);
 
-    /* submailboxes - we're using internal names here */
-    mboxlist_findall(NULL, pattern, 1, 0, 0, mboxlist_changequota, (void *)root);
+	/* submailboxes - we're using internal names here */
+	mboxlist_findall(NULL, pattern, 1, 0, 0, mboxlist_changequota, (void *)root);
+    }
 
 done:
-    if (tid) quota_abort(&tid);
     if (!r) sync_log_quota(root);
 
     return r;
@@ -2516,12 +2405,14 @@ void mboxlist_open(const char *fname)
     int ret, flags;
     char *tofree = NULL;
 
-    if (!fname)
-	fname = config_getstring(IMAPOPT_MBOXLIST_DB_PATH);
-
     /* create db file name */
     if (!fname) {
-	tofree = strconcat(config_dir, FNAME_MBOXLIST, (char *)NULL);
+	size_t fname_len = strlen(config_dir)+strlen(FNAME_MBOXLIST)+1;
+
+	tofree = xmalloc(fname_len);
+	strlcpy(tofree, config_dir, fname_len);
+	strlcat(tofree, FNAME_MBOXLIST, fname_len);
+
 	fname = tofree;
     }
 
@@ -2539,7 +2430,7 @@ void mboxlist_open(const char *fname)
 	fatal("can't read mailboxes file", EC_TEMPFAIL);
     }    
 
-    free(tofree);
+    if (tofree) free(tofree);
 
     mboxlist_dbopen = 1;
 }
@@ -2607,7 +2498,7 @@ static void mboxlist_closesubs(struct db *sub)
  * is the user's login id.  For each matching mailbox, calls
  * 'proc' with the name of the mailbox.
  */
-int mboxlist_findsub(struct namespace *namespace,
+int mboxlist_findsub(struct namespace *namespace __attribute__((unused)),
 		     const char *pattern, int isadmin __attribute__((unused)),
 		     const char *userid, struct auth_state *auth_state, 
 		     int (*proc)(), void *rock, int force)
@@ -2625,8 +2516,6 @@ int mboxlist_findsub(struct namespace *namespace,
     char domainpat[MAX_MAILBOX_BUFFER]; /* do intra-domain fetches only */
     char *pat = NULL;
 
-    if (!namespace) namespace = mboxname_get_adminnamespace();
-
     if (config_virtdomains && userid && (p = strchr(userid, '@'))) {
 	userlen = p - userid;
 	domainlen = strlen(p); /* includes separator */
@@ -2645,8 +2534,6 @@ int mboxlist_findsub(struct namespace *namespace,
     cbrock.checkshared = 0;
     cbrock.proc = proc;
     cbrock.procrock = rock;
-    cbrock.prev = NULL;
-    cbrock.prevlen = 0;
 
     /* open the subscription file that contains the mailboxes the 
        user is subscribed to */
@@ -2731,9 +2618,6 @@ int mboxlist_findsub(struct namespace *namespace,
 		       usermboxname, usermboxnamelen,
 		       &find_p, &find_cb, &cbrock,
 		       NULL);
-	free(cbrock.prev);
-	cbrock.prev = NULL;
-	cbrock.prevlen = 0;
 
 	cbrock.usermboxname = usermboxname;
 	cbrock.usermboxnamelen = usermboxnamelen;
@@ -2742,25 +2626,20 @@ int mboxlist_findsub(struct namespace *namespace,
 	cbrock.usermboxnamelen = 0;
     }
 
-    if (isadmin || namespace->accessible[NAMESPACE_USER]) {
-	cbrock.find_namespace = NAMESPACE_USER;
-	/* switch to pattern with domain prepended */
-	glob_free(&cbrock.g);
-	cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
-	cbrock.inboxoffset = 0;
-	if (usermboxnamelen) {
-	    usermboxname[--usermboxnamelen] = '\0';
-	    cbrock.usermboxname = usermboxname;
-	    cbrock.usermboxnamelen = usermboxnamelen;
-	}
-	/* search for all remaining mailboxes.
-	   just bother looking at the ones that have the same pattern prefix. */
-	SUBDB->foreach(subs, domainpat, domainlen + prefixlen,
-		       &find_p, &find_cb, &cbrock, NULL);
-	free(cbrock.prev);
-	cbrock.prev = NULL;
-	cbrock.prevlen = 0;
-   }
+    cbrock.find_namespace = NAMESPACE_USER;
+    /* switch to pattern with domain prepended */
+    glob_free(&cbrock.g);
+    cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
+    cbrock.inboxoffset = 0;
+    if (usermboxnamelen) {
+	usermboxname[--usermboxnamelen] = '\0';
+	cbrock.usermboxname = usermboxname;
+	cbrock.usermboxnamelen = usermboxnamelen;
+    }
+    /* search for all remaining mailboxes.
+       just bother looking at the ones that have the same pattern prefix. */
+    SUBDB->foreach(subs, domainpat, domainlen + prefixlen, 
+		   &find_p, &find_cb, &cbrock, NULL);
 
   done:
     if (subs) mboxlist_closesubs(subs);
@@ -2770,22 +2649,6 @@ int mboxlist_findsub(struct namespace *namespace,
     return r;
 }
 
-int mboxlist_allsubs(const char *userid, foreach_cb *proc, void *rock)
-{
-    struct db *subs = NULL;
-    int r;
-
-    /* open subs DB */
-    r = mboxlist_opensubs(userid, &subs);
-    if (r) return r;
-
-    r = SUBDB->foreach(subs, "", 0, NULL, proc, rock, 0);
-
-    mboxlist_closesubs(subs);
-
-    return r;
-}
-
 int mboxlist_findsub_alt(struct namespace *namespace,
 			 const char *pattern, int isadmin __attribute__((unused)),
 			 const char *userid, struct auth_state *auth_state, 
@@ -2804,8 +2667,6 @@ int mboxlist_findsub_alt(struct namespace *namespace,
     char domainpat[MAX_MAILBOX_BUFFER]; /* do intra-domain fetches only */
     char *pat = NULL;
 
-    if (!namespace) namespace = mboxname_get_adminnamespace();
-
     if (config_virtdomains && userid && (p = strchr(userid, '@'))) {
 	userlen = p - userid;
 	domainlen = strlen(p); /* includes separator */
@@ -2824,8 +2685,6 @@ int mboxlist_findsub_alt(struct namespace *namespace,
     cbrock.checkshared = 0;
     cbrock.proc = proc;
     cbrock.procrock = rock;
-    cbrock.prev = NULL;
-    cbrock.prevlen = 0;
 
     /* open the subscription file that contains the mailboxes the 
        user is subscribed to */
@@ -2883,10 +2742,11 @@ int mboxlist_findsub_alt(struct namespace *namespace,
      * Append pattern to "INBOX.", search for those subscriptions next
      */
     if (userid) {
-	strlcpy(patbuf, usermboxname, sizeof(patbuf));
+	strlcpy(patbuf, "INBOX.", sizeof(patbuf));
 	strlcat(patbuf, pattern, sizeof(patbuf));
-	cbrock.g = glob_init(patbuf, GLOB_HIERARCHY);
-	cbrock.inboxoffset = 0;
+	cbrock.g = glob_init(patbuf, GLOB_HIERARCHY|GLOB_INBOXCASE);
+	cbrock.inboxcase = glob_inboxcase(cbrock.g);
+	cbrock.inboxoffset = domainlen+userlen;
 	cbrock.find_namespace = NAMESPACE_INBOX;
 
 	/* iterate through prefixes matching usermboxname */
@@ -2894,9 +2754,6 @@ int mboxlist_findsub_alt(struct namespace *namespace,
 		       usermboxname, usermboxnamelen,
 		       &find_p, &find_cb, &cbrock,
 		       NULL);
-	free(cbrock.prev);
-	cbrock.prev = NULL;
-	cbrock.prevlen = 0;
 
 	glob_free(&cbrock.g);
 
@@ -2918,39 +2775,33 @@ int mboxlist_findsub_alt(struct namespace *namespace,
      *
      * If "Other Users*" can match pattern, search for those subscriptions next
      */
+    len = strlen(namespace->prefix[NAMESPACE_USER]);
+    if(len>0) len--; /* Remove Separator */
+    if (!strncmp(namespace->prefix[NAMESPACE_USER], pattern,
+		 prefixlen < len ? prefixlen : len)) {
 
-    if (isadmin || namespace->accessible[NAMESPACE_USER]) {
-	len = strlen(namespace->prefix[NAMESPACE_USER]);
-	if(len>0) len--; /* Remove Separator */
-	if (!strncmp(namespace->prefix[NAMESPACE_USER], pattern,
-		     prefixlen < len ? prefixlen : len)) {
-
-	    if (prefixlen < len) {
-		strlcpy(domainpat+domainlen, pattern+prefixlen,
-			sizeof(domainpat)-domainlen);
-		cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
-	    }
-	    else {
-		strlcpy(domainpat+domainlen, "user",
-			sizeof(domainpat)-domainlen);
-		strlcat(domainpat, pattern+len, sizeof(domainpat));
-		cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
-	    }
-	    cbrock.find_namespace = NAMESPACE_USER;
-	    cbrock.inboxoffset = 0;
-	
-	    /* iterate through prefixes matching usermboxname */
-	    strlcpy(domainpat+domainlen, "user", sizeof(domainpat)-domainlen);
-	    SUBDB->foreach(subs,
-			   domainpat, strlen(domainpat),
-			   &find_p, &find_cb, &cbrock,
-			   NULL);
-	    free(cbrock.prev);
-	    cbrock.prev = NULL;
-	    cbrock.prevlen = 0;
-
-	    glob_free(&cbrock.g);
+	if (prefixlen < len) {
+	    strlcpy(domainpat+domainlen, pattern+prefixlen,
+		    sizeof(domainpat)-domainlen);
+	    cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
 	}
+	else {
+	    strlcpy(domainpat+domainlen, "user",
+		   sizeof(domainpat)-domainlen);
+	    strlcat(domainpat, pattern+len, sizeof(domainpat));
+	    cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
+	}
+	cbrock.find_namespace = NAMESPACE_USER;
+	cbrock.inboxoffset = 0;
+	
+	/* iterate through prefixes matching usermboxname */
+	strlcpy(domainpat+domainlen, "user", sizeof(domainpat)-domainlen);
+	SUBDB->foreach(subs,
+		       domainpat, strlen(domainpat),
+		       &find_p, &find_cb, &cbrock,
+		       NULL);
+
+	glob_free(&cbrock.g);
     }
 
     /*
@@ -2959,63 +2810,55 @@ int mboxlist_findsub_alt(struct namespace *namespace,
      * search for all remaining subscriptions.
      * just bother looking at the ones that have the same pattern prefix.
      */
-    if (isadmin || namespace->accessible[NAMESPACE_SHARED]) {
-	len = strlen(namespace->prefix[NAMESPACE_SHARED]);
-	if(len>0) len--; /* Remove Separator */
-	if (!strncmp(namespace->prefix[NAMESPACE_SHARED], pattern,
-		     prefixlen < len ? prefixlen : len)) {
+    len = strlen(namespace->prefix[NAMESPACE_SHARED]);
+    if(len>0) len--; /* Remove Separator */
+    if (!strncmp(namespace->prefix[NAMESPACE_SHARED], pattern,
+		 prefixlen < len ? prefixlen : len)) {
 
-	    cbrock.find_namespace = NAMESPACE_SHARED;
-	    cbrock.inboxoffset = 0;
+	cbrock.find_namespace = NAMESPACE_SHARED;
+	cbrock.inboxoffset = 0;
 
-	    if (prefixlen <= len) {
-		/* Skip pattern which matches shared namespace prefix */
-		for (p = pat+prefixlen; *p; p++) {
-		    if (*p == '%') continue;
-		    else if (*p == '.') p++;
-		    break;
-		}
-
-		if (*pattern && !strchr(pattern, '.') &&
-		    pattern[strlen(pattern)-1] == '%') {
-		    /* special case:  LSUB "" *% -- output prefix */
-		    cbrock.checkshared = 1;
-		}
-
-		if ((cbrock.checkshared || prefixlen == len) && !*p) {
-		    /* special case:  LSUB "" % -- output prefix
-		       (if we have a shared mbox) and quit */
-		    strlcpy(domainpat+domainlen, "*", sizeof(domainpat)-domainlen);
-		    cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
-		    cbrock.checkshared = 2;
-		}
-		else {
-		    strlcpy(domainpat+domainlen, p, sizeof(domainpat)-domainlen);
-		    cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
-		}
-
-		domainpat[domainlen] = '\0';
-		SUBDB->foreach(subs,
-			       domainpat, domainlen,
-			       &find_p, &find_cb, &cbrock,
-			       NULL);
-		free(cbrock.prev);
-		cbrock.prev = NULL;
-		cbrock.prevlen = 0;
+	if (prefixlen <= len) {
+	    /* Skip pattern which matches shared namespace prefix */
+	    for (p = pat+prefixlen; *p; p++) {
+		if (*p == '%') continue;
+		else if (*p == '.') p++;
+		break;
 	    }
-	    else if (pattern[len] == '.') {
-		strlcpy(domainpat+domainlen, pattern+len+1,
-		        sizeof(domainpat)-domainlen);
+
+	    if (*pattern && !strchr(pattern, '.') &&
+		pattern[strlen(pattern)-1] == '%') {
+		/* special case:  LSUB "" *% -- output prefix */
+  		cbrock.checkshared = 1;
+	    }
+
+	    if ((cbrock.checkshared || prefixlen == len) && !*p) {
+		/* special case:  LSUB "" % -- output prefix
+		   (if we have a shared mbox) and quit */
+		strlcpy(domainpat+domainlen, "*", sizeof(domainpat)-domainlen);
 		cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
-
-		SUBDB->foreach(subs,
-			       domainpat, domainlen+prefixlen-(len+1),
-			       &find_p, &find_cb, &cbrock,
-			       NULL);
-		free(cbrock.prev);
-		cbrock.prev = NULL;
-		cbrock.prevlen = 0;
+		cbrock.checkshared = 2;
 	    }
+	    else {
+		strlcpy(domainpat+domainlen, p, sizeof(domainpat)-domainlen);
+		cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
+	    }
+
+	    domainpat[domainlen] = '\0';
+	    SUBDB->foreach(subs,
+			   domainpat, domainlen,
+			   &find_p, &find_cb, &cbrock,
+			   NULL);
+	}
+	else if (pattern[len] == '.') {
+	    strlcpy(domainpat+domainlen, pattern+len+1,
+		    sizeof(domainpat)-domainlen);
+	    cbrock.g = glob_init(domainpat, GLOB_HIERARCHY);
+
+	    SUBDB->foreach(subs,
+			   domainpat, domainlen+prefixlen-(len+1),
+			   &find_p, &find_cb, &cbrock,
+			   NULL);
 	}
     }
 
