@@ -119,7 +119,7 @@ int mboxlist_createsync(const char *name, int mbtype,
 			const char *partition, 
 			const char *userid, struct auth_state *auth_state,
 			int options, unsigned uidvalidity, const char *acl,
-			const char *uniqueid, struct mailbox **mboxptr);
+			const char *uniqueid);
 
 /* delated delete */
 /* Translate delete into rename */
@@ -155,19 +155,24 @@ int mboxlist_sync_setacls(const char *name, const char *acl);
 int mboxlist_findall(struct namespace *namespace,
 		     const char *pattern, int isadmin, const char *userid, 
 		     struct auth_state *auth_state, int (*proc)(), void *rock);
+int mboxlist_findall_std(struct namespace *namespace,
+			 const char *pattern, int isadmin, const char *userid, 
+			 struct auth_state *auth_state, int (*proc)(),
+			 void *rock);
 int mboxlist_findall_alt(struct namespace *namespace,
 			 const char *pattern, int isadmin, const char *userid, 
 			 struct auth_state *auth_state, int (*proc)(),
 			 void *rock);
 
-/* direct access to subs DB */
-int mboxlist_allsubs(const char *userid, foreach_cb *proc, void *rock);
-
 /* Find subscribed mailboxes that match 'pattern'. */
 int mboxlist_findsub(struct namespace *namespace,
 		     const char *pattern, int isadmin, const char *userid, 
 		     struct auth_state *auth_state, int (*proc)(), void *rock,
 		     int force);
+int mboxlist_findsub_std(struct namespace *namespace,
+			 const char *pattern, int isadmin, const char *userid, 
+			 struct auth_state *auth_state, int (*proc)(),
+			 void *rock, int force);
 int mboxlist_findsub_alt(struct namespace *namespace,
 			 const char *pattern, int isadmin, char const *userid, 
 			 struct auth_state *auth_state, int (*proc)(),
