@@ -1,6 +1,6 @@
 /* message.h -- Message parsing
  *
- * Copyright (c) 1994-2010 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1994-2008 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -58,78 +58,6 @@
 #include "prot.h"
 #include "mailbox.h"
 
-/* cyrus.cache file item buffer */
-struct ibuf {
-    char *start, *end, *last;
-};
-
-/*
- * Parsed form of a body-part
- */
-struct body {
-    /* Content-* header information */
-    char *type;
-    char *subtype;
-    struct param *params;
-    char *id;
-    char *description;
-    char *encoding;
-    char *md5;
-    char *disposition;
-    struct param *disposition_params;
-    struct param *language;
-    char *location;
-
-    /* Location/size information */
-    long header_offset;
-    long header_size;
-    long header_lines;
-    long content_offset;
-    long content_size;
-    long content_lines;
-    long boundary_size;		/* Size of terminating boundary */
-    long boundary_lines;
-
-    int numparts;		/* For multipart types */
-    struct body *subpart;	/* For message/rfc822 and multipart types */
-
-    /*
-     * Other header information.
-     * Only meaningful for body-parts at top level or
-     * enclosed in message/rfc-822
-     */
-    char *date;
-    char *subject;
-    struct address *from;
-    struct address *sender;
-    struct address *reply_to;
-    struct address *to;
-    struct address *cc;
-    struct address *bcc;
-    char *in_reply_to;
-    char *message_id;
-    char *received_date;
-
-    /*
-     * Cached headers.  Only filled in at top-level
-     */
-    struct ibuf cacheheaders;
-
-    /*
-     * decoded body.  Filled in as needed.
-     */
-    char *decoded_body;
-
-    /* Message GUID. Only filled in at top level */
-    struct message_guid guid;
-};
-
-/* List of Content-type parameters */
-struct param {
-    struct param *next;
-    char *attribute;
-    char *value;
-};
 extern int message_copy_strict P((struct protstream *from, FILE *to,
 				  unsigned size, int allow_null));
 
