@@ -59,7 +59,9 @@ int main(int argc, char *argv[])
     int txnp = 0;
     int r;
 
+    printf("Initing enviornment in '.'...\n");
     TRY(DB->init(".", 0));
+    printf("Ready!\n");
 
     for (;;) {
 	if (fgets(buf, sizeof buf, stdin) == NULL) break;
@@ -68,15 +70,15 @@ int main(int argc, char *argv[])
 	    char *fname = buf + 5;
 
 	    if (db) { /* close it */
-		TRY(DB->close(db));
+		TRY((DB->close)(db));
 	    }
-	    TRY(DB->open(fname, &db));
+	    TRY((DB->open)(fname, 1, &db));
 
 	    printf("ok\n");
 	} else if (!db) {
 	    TRY(db == NULL);
 	} else if (!strncasecmp(buf, "close", 5)) {
-	    TRY(DB->close(db));
+	    TRY((DB->close)(db));
 	    db = NULL;
 	    printf("ok\n");
 	} else if (!strncasecmp(buf, "put ", 4)) {
